import sys
import os
import shutil

class VeriteemInstall():

      installPath = None

      def __init__(self):
           
          for path in sys.path:
              if path == os.getcwd() :
                 continue
              if os.path.isdir(path) and "veriteem" in os.listdir(path):
                 break

          VeriteemInstall.installPath = os.path.join(path, "veriteem")

      #
      #  Check whether we need to update the ethereum go installation
      #  and rebuild the veriteem executable
      #
      @classmethod
      def CheckInstallation(self):
          #
          # First check if the <package install>/bin/veriteem executable exists
          #
          exFile = os.path.join(VeriteemInstall.installPath, "bin", "veriteem")
          print("exFile " + exFile)
          if not os.path.isfile(exFile) :
             print("Please wait... need to install assets and programs")
             VeriteemInstall.install()
             return

          #
          # We have a veriteem executable, check its version
          #
          os.chdir(VeriteemInstall.installPath)
  
          fp = open("VERSION", "r")
          version = fp.read()
          fp.close()
          version = version.rstrip()

          for filename in os.listdir(".."):
              if filename.find("veriteem-") != -1 :
                 break

          sidx   = filename.find('-') + 1
          eidx   = filename.find('.dist')   
          if eidx < 0 :
             eidx = filename.find('-py')
          pversion = filename[sidx:eidx]
          print("version " + version)
          print("pversion " + pversion)
          if pversion != version :
             print ("Please wait, need to update package")
             VeriteemInstall.install()
      #
      #  Do the full installation setup
      #  
      @classmethod
      def install(self):
          #
          #  Install go to the standard PATH environment variable
          #
          VeriteemInstall.addGoToPath()
          VeriteemInstall.installEthereum()

      def find_module_path():
          for path in sys.path:
              if path == os.getcwd() :
                 continue
              if os.path.isdir(path) and "veriteem" in os.listdir(path):
                 return os.path.join(path, "veriteem")
          return None
      #
      #  Add Golang to the executable path
      #  
      @classmethod
      def addGoToPath(self):
          try:
             with open("/etc/environment", "r") as envFile:
                  lines = envFile.readlines()
             envFile.close()
          except:
             print("Cannot modify /etc/environment to set PATH for go")
             return

          #
          #  Find the PATH= specification in the environment file
          #
          for idx in range(len(lines)) :
              pathSet = lines[idx].find("PATH=")
              if pathSet >= 0 :
                 break
                    
          #
          #  add /usr/lib/go-1.9/bin to the path if not present
          #
          if pathSet < 0 :
             print("PATH not found /etc/environment ")
             return
             
          goPath = lines[idx].find("go-1.9")
          if goPath >= 0 :
             print("go is defined in PATH ")
             return

          pathStr = lines[idx].rstrip()
          pathStr = pathStr.replace("'", '')
          pathStr = pathStr.replace('"', '')
          
          newPath = pathStr + ":/usr/lib/go-1.9/bin"
          print("PATH: " + newPath)
                 
          try:
             envFile = open("/etc/environment", "w")
          except:
             print("Unable to open /etc/environment to update PATH")
             return

          for jdx in range(len(lines)) :
              if jdx != idx:
                 envFile.write(lines[jdx])
              if jdx == idx:
                 envFile.write(newPath + "\n")
              
          envFile.close()
 
           
      #
      #  Download go-ethereum 
      #  
      @classmethod
      def installEthereum(self):

          try:
              path = os.path.join(VeriteemInstall.installPath, "bin")
              print("installgo is in " + path)
              os.chdir(path)
              os.system("./installgo.sh")
          except:
              print("Unable to install ethereum")
              return

          






#!/usr/bin/python3
import traceback
import sys
import os
import argparse
import veriteem as vmx


def main(argv):

    parser = argparse.ArgumentParser()
    parser.add_argument("-p","--path",    help="configuration path")
    parser.add_argument("-t","--tools",   help="access Veriteem javascript tools ", action="store_true")
    parser.add_argument("-z","--zigbee",  help="access Veriteem ZigBee tools ", action="store_true")


    path    = None

    args     = parser.parse_args()
    if args.path != None:
       path = args.path


    try:
       myConfig = vmx.Config(path)
       myConfig.LoadConfig()
    except:
       raise Exception("Site Configuration Not Specified")

    if args.tools == True:
       try:
           scriptPath = os.path.join(myConfig.INSTALLPATH, "scripts")
           os.chdir(scriptPath)
       except:
           ErrMsg = "Cannot locate tools in " + scriptPath
           raise Exception(ErrMsg)
           
    if args.zigbee == True:
       try:
           ledgerPath = myConfig.find_module_path("veriteemcomplianceledger")
           scriptPath = os.path.join(ledgerPath, "scripts")
           os.chdir(scriptPath)
       except:
           ErrMsg = "Cannot locate tools in " + scriptPath
           raise Exception(ErrMsg)
           
    gethExe = myConfig.getChainExe()
    Cmd = gethExe + " attach ipc:" + myConfig.GETHDATA + "/geth.ipc --datadir " + myConfig.GETHDATA
    os.system(Cmd)
  
if __name__ == "__main__":
  main(sys.argv)



import sys
import os
import shutil


from .Config import Config 

class InitGeth():
   
    myConfig = []

    def __init__(self, path):

       
       try:
          InitGeth.myConfig = Config(path)
          InitGeth.myConfig.LoadConfig()
       except:
          raise RunTimeError("Site Configuration Not Specified")
       

    @classmethod
    def CreateBlockChain(self):

        #
        # Make sure we have some necessary files before making changes
        #
        filePath = os.path.join(InitGeth.myConfig.CONFIGPATH, "genesis.json")
        if not os.path.exists(filePath):
           raise RunTimeError ("Missing genesis.json")

        #
        #  if we have an existing installation, remove it
        #
        filePath = os.path.join(InitGeth.myConfig.CONFIGPATH, "GethData")
        if os.path.exists(filePath):
           try:
              shutil.rmtree(filePath)
           except Exception as ex:
              print(ex)

        #
        #  Create the directories needed for operations
        #
        try:
           os.mkdir(filePath)
           os.mkdir(filePath + "/logs")
           os.mkdir(filePath + "/geth")
        except Exception as ex:
           print (ex)

        #
        # Use our modified geth to generate the new block chain with the genesis file
        #
        try:
           logFile = os.path.join(InitGeth.myConfig.GETHDATA, "logs", "geth.log")
           chainExe  = InitGeth.myConfig.getChainExe()
           genesis = os.path.join(InitGeth.myConfig.CONFIGPATH, "genesis.json")
           Cmd = chainExe + " --datadir " + InitGeth.myConfig.GETHDATA + " init " + genesis +  " >>" + logFile + " 2>&1 "
           print(Cmd)
           os.system(Cmd)
        except Exception as ex:
           print (ex)






import traceback
import sys
import os
import shutil
import time

from .Config import Config 

class StartMiner():

    myConfig = []

    def __init__(self, path):

        try:
           StartMiner.myConfig = Config(path)
        except:
           print ("Site configuration not specified")
           print(traceback.format_exc())
           return;
        try:
           StartMiner.myConfig.LoadConfig()
        except:
           print(traceback.format_exc())

    @classmethod
    def Start(self):
        #
        #  We are running our modified geth
        #
        pwFile = open("pauth.txt", "w")
        passwd = StartMiner.myConfig.ACCOUNTPWD 
        pwFile.writelines(passwd)
        pwFile.close()
        chainExe = StartMiner.myConfig.getChainExe()
        Cmd = chainExe + ' --mine --rpc --rpcaddr localhost --rpcport 8545 --rpcapi "web3,eth" --rpccorsdomain "http://localhost:8000" '
        Cmd = Cmd + '--datadir ' + StartMiner.myConfig.GETHDATA  + ' '
    
        Cmd = Cmd + '--port 60303 --networkid ' + StartMiner.myConfig.NETWORK + ' --targetgaslimit 15000000 --gasprice 0 --maxpeers 25 --nat none '
    
        Cmd = Cmd + ' --keystore ' + StartMiner.myConfig.KEYSTORE + ' --unlock ' + StartMiner.myConfig.ACCOUNT + ' --etherbase ' + StartMiner.myConfig.ACCOUNT 
        Cmd = Cmd + ' --password pauth.txt --bootnodes ' + StartMiner.myConfig.BOOTNODE
        Cmd = Cmd +  ' >> ' + os.path.join(StartMiner.myConfig.GETHDATA, 'logs', 'geth.log') +  ' 2>&1 &'
        
        print (Cmd)
        os.system(Cmd)
    
        print("Waiting for server to start")
        time.sleep(10)
        os.remove("pauth.txt")



#! python
import traceback
import sys
import os
import argparse
import veriteem as vmx


def main(argv):

    parser = argparse.ArgumentParser()
    parser.add_argument("-c","--config", help="configure parameters",  action="store_true")
    parser.add_argument("-i","--init",   help="initialize blockchain", action="store_true")
    parser.add_argument("-r","--run",    help="start the veriteem service ", action="store_true")
    parser.add_argument("-m","--miner",  help="start the veriteem mining service ", action="store_true")
    parser.add_argument("-s","--stop",   help="stop the chain miner",  action="store_true")
    parser.add_argument("-a","--access", help="add account to access control", action="store_true")
    parser.add_argument("-t","--total",  help="configure, init, and start the chain miner", action="store_true")
    parser.add_argument("-p","--path",   help="configuration path")
    parser.add_argument("-n","--nosave", help="do not save path", action="store_true")

    path    = None

    args     = parser.parse_args()

    #
    #  Check on the installation to see if we need to install assets/programs
    #
    myInstall = vmx.VeriteemInstall()
    myInstall.CheckInstallation()

    # if no arguments were provided, default to --total
    if len(argv) == 1 :
       args.total = True

    if args.path != None:
       path     = args.path

    # if the user wants to run all items or the config option
    if (args.config == True) or (args.total == True) :
       try:
          myConfig = vmx.Configure(path, args.nosave )
       except Exception as ex:
          # print(traceback.format_exc())
          print(ex)
          return
       myConfig.Create()

    # if the user wants to run all items or create the blockchain
    if (args.init == True) or (args.total == True) :
       try:
          myInit = vmx.InitGeth(path)
       except:
          print(traceback.format_exc())
          return
       myInit.CreateBlockChain()

    # if the user wants to run all items or start the veriteem service
    if (args.run == True) or (args.total == True) :
       try:
          myVeriteem = vmx.StartVeriteem(path)
       except:
          print(traceback.format_exc())
          return
       myVeriteem.Start()

    # if the user wants to start the miner
    if (args.miner == True)  :
       try:
          myMiner = vmx.StartMiner(path)
       except:
          print(traceback.format_exc())
          return
       myMiner.Start()

    if (args.access == True) or (args.total == True) :
       try:
          myAccess = vmx.AccessControl(path)
       except:
          print(traceback.format_exc())
          return
       myAccess.AddAccess()

    if args.total == True :
       readMe = myConfig.myConfig.getPackageFile("veriteem", None, "README.md")
       fd = open(readMe, "r")
       readMeStr = fd.read()
       print(readMeStr)
       
    if args.stop == True :
       Cmd = "killall veriteem"
       os.system(Cmd)
  
if __name__ == "__main__":
  main(sys.argv)



import traceback
import sys
import os
import shutil
import time

from .Config import Config 

class StartVeriteem():

    myConfig = []

    def __init__(self, path):

        try:
           StartVeriteem.myConfig = Config(path)
        except:
           print ("Site configuration not specified")
           print(traceback.format_exc())
           return;
        try:
           StartVeriteem.myConfig.LoadConfig()
        except:
           print(traceback.format_exc())

    @classmethod
    def Start(self):
        #
        #  We are running our modified geth
        #
        chainExe = StartVeriteem.myConfig.getChainExe()
        Cmd = chainExe + ' --rpc --rpcaddr localhost --rpcport 8545 --rpcapi "web3,eth" --rpccorsdomain "http://localhost:8000" '
        Cmd = Cmd + '--datadir ' + StartVeriteem.myConfig.GETHDATA  + ' '
    
        Cmd = Cmd + '--port 60303 --networkid ' + StartVeriteem.myConfig.NETWORK + ' --targetgaslimit 15000000 --gasprice 0 --maxpeers 25 --nat none '
    
        Cmd = Cmd + ' --bootnodes ' + StartVeriteem.myConfig.BOOTNODE
        Cmd = Cmd +  ' >> ' + os.path.join(StartVeriteem.myConfig.GETHDATA, 'logs', 'geth.log') +  ' 2>&1 &'
        
        print (Cmd)
        os.system(Cmd)
    
        print("Waiting for server to start")
        time.sleep(10)



import traceback
import sys
import os
import json
import base64

class Config():

    INSTALLPATH = None
    CONFIGPATH  = None

    def __init__(self, path):
        Config.INSTALLPATH = self.find_module_path("veriteem")
        Config.CONFIGPATH  = self.getConfigPath(path)
        
        
    @classmethod
    def LoadConfig(self):
        if Config.CONFIGPATH is None:
           raise Exception("Must specify directory in which to store app data to proceed")

        #
        #  If a Config.json file exists, pull it in for default settings
        #
        filePath = self.getFilePath("Config.json")
        print("ConfigFile = " + filePath)
        try:
            ConfigFile = open(filePath)
            ConfigStr  = ConfigFile.read()
            ConfigData = json.loads(ConfigStr)
            ConfigFile.close()
        except Exception:
            print("Failed to read config file")
            ConfigData = []
            
        try:
            Config.SITEID = ConfigData["SITEID"]
        except:
            Config.SITEID = "NONAME"
   
        try:
            Config.GETHDATA = ConfigData["GETHDATA"]
        except:
            Config.GETHDATA = os.path.join(Config.CONFIGPATH , "GethData")

        try:
            Config.BOOTNODE = ConfigData["BOOTNODE"]
        except:
            Config.BOOTNODE = None

        try:
            Config.KEYSTORE = ConfigData["KEYSTORE"]
        except:
            Config.KEYSTORE = None
   
        try:
            Config.NETWORK = ConfigData["NETWORK"]
        except:
            Config.NETWORK = None
      
        try:
            pwd = ConfigData["ACCOUNTPWD"]
            Config.ACCOUNTPWD = self.decode("VmxSanDiego", pwd)
        except:
            Config.ACCOUNTPWD = None

        try:
            Config.ACCOUNT = ConfigData["ACCOUNT"]
        except:
            Config.ACCOUNT = None


    @classmethod
    def getFilePath(self, fileName):
        filePath = os.path.join(Config.CONFIGPATH, fileName)
        if os.path.isfile(filePath) :
           return filePath

        filePath = os.path.join(Config.INSTALLPATH, "assets", fileName)
        if os.path.isfile(filePath) :
           return filePath
        return None

    @classmethod
    def find_module_path(self, packageName):
        for path in sys.path:
            if path != os.getcwd() :
               if os.path.isdir(path) and packageName in os.listdir(path):
                  return os.path.join(path, packageName)


    @classmethod
    def getCookiePath(self):
        try:
           homeDir = os.environ["HOME"]
        except:
           print("HOME environment variable not set")
           return None

        homeDir = homeDir + "/.veriteem"
        if os.path.isdir(homeDir) == False:
           print(homeDir + " does not exist")
           return None

        try:
           configFile = os.path.join(homeDir, "config")
           cookieFile = open(configFile, "r")
           path = cookieFile.read()
           cookieFile.close()
        except:
           print("Error reading " + homeDir)
           path = None

        return path

    @classmethod
    def getConfigPath(self,path):
        #
        #  See if we have the cookie that holds the location of the
        #  app data
        #
        if path is None:
           path = self.getCookiePath()
        #
        #  If a Config.json file exists, pull it in for default settings
        #
    
        if path is None:
           cwd = os.getcwd()
           path = input("What directory should Veriteem reside in? [" + cwd + "] : ")
           if not path:
              path = cwd

        if os.path.isdir(path)  == False :
           response = input(path + " Does not exist. Create it? [Y] : ")
           if not response:
              response = "Y"
           if response.upper() == "N" : 
              raise Exception("Need valid directory to store app data in") 
           os.mkdir(path)
           
        return path

    @classmethod
    def getChainExe(self):
        path = os.path.join(Config.INSTALLPATH, "bin/veriteem")
        return path 

    @classmethod
    def decode(self, key, enc):
        dec = []
        enc = base64.urlsafe_b64decode(enc).decode()
        for i in range(len(enc)):
            key_c = key[i % len(key)]
            dec_c = chr((256 + ord(enc[i]) - ord(key_c)) % 256)
            dec.append(dec_c)
        return "".join(dec)

    @classmethod
    def getPackageFile(self, packageName, subDirectory, fileName):
        packagePath = Config.find_module_path(packageName) 
        if packagePath == None :
           return None
        if subDirectory == None :
           path = os.path.join(packagePath,fileName)
        else :
           path = os.path.join(packagePath,subDirectory,fileName)
        return path



#!/usr/bin/env python
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import atexit
import os
import sys
import shutil

from setuptools import setup
from setuptools import find_packages
from setuptools.command.install import install

setup(
    name="veriteem",
    version="0.1.13",
    packages=find_packages('src'),
    package_dir={'':'src'},

    package_data={
         # include any asset files found in the 'veriteem' package:
        'veriteem': ['README.md', 'assets/*', 'bin/*', 'scripts/*' ],
    },
    scripts=['src/veriteem/VeriteemConfig.py',
             'src/veriteem/Veriteem.py',
            ],

    # metadata for upload to PyPI
    author="Brad Ree",
    author_email="bree@verimatrix.com",
    description="Veriteem Blockchain DLT",
    license="GPL",
    keywords="Verimatrix Blockchain Compliance Ledger ",
    url="https://github.com/VerimatrixGen1/Veriteem",   # project home page, if any
    project_urls={
        "Bug Tracker": "https://bugs.example.com/HelloWorld/",
        "Documentation": "https://github.com/VerimatrixGen1/Veriteem/wiki",
        "Source Code": "https://github.com/VerimatrixGen1/Veriteem",
    }

    # could also include long_description, download_url, classifiers, etc.  
)



import sys
import os
import shutil
import getpass
import csv
import json
import base64

from .Config import Config

class Configure():

    noSavePath = False
    AccountPwd = None
    Account    = None
    myConfig   = []
  
    def __init__(self, path, noSavePath):
        #
        #  locate the directory where the config.json exists
        # 
        Configure.noSavePath = noSavePath 
        Configure.myConfig = Config(path)
        try:
           Configure.myConfig.LoadConfig() 
        except Exception as ex:
           raise Exception(ex)
        
    @classmethod
    def getChainId(self):
        try:
           filePath    = Configure.myConfig.getFilePath("genesis.json")
           genesisFile = open(filePath, "r") 
           genesisStr  = genesisFile.read()
           genesisData = json.loads(genesisStr)
        except:
           raise Exception("Cannot locate valid genesis.json file")

        try:
           ConfigData = genesisData["config"]
           chainId = ConfigData["chainId"]
        except:
           raise Exception("no chainId value in genesis.json")
        return chainId

    @classmethod
    def Create(self):
        try:
           chainId = Configure.getChainId()
        except:
           raise Exception("Cannot locate genesis.json file")

        DefaultKeystore = os.path.join(Configure.myConfig.CONFIGPATH, "KeyFile")

        Result = input("Use Generic Defaults? [Y] : ")
        if not Result:
           Result = 'Y'
        else:
           Result = Result.upper()
   
        if Result.upper() == "N":

           GethData = Configure.myConfig.GETHDATA
           if GethData is None:
               GethData = Configure.myConfig.getFilePath("GethData")

           KeyStore = Configure.myConfig.KEYSTORE 
           if KeyStore is None:
               KeyStore = DefaultKeystore

           BootNode = Configure.myConfig.BOOTNODE
           if BootNode is None:
               BootNode = "enode://0949976132f8446dfa40023bd0c148d0d7991f67f8b06570f8455194b54effb16d9476046a4f46dd29a33b34a8b1c396e80fcac8e30101160807a22cfc985f86@34.199.143.211:60303"  
           Configure.Account = Configure.myConfig.ACCOUNT
           Configure.AccountPwd = Configure.myConfig.ACCOUNTPWD
         
        else:
           try:
               BootNode = Configure.myConfig.BOOTNODE
           except:
               BootNode = "enode://0949976132f8446dfa40023bd0c148d0d7991f67f8b06570f8455194b54effb16d9476046a4f46dd29a33b34a8b1c396e80fcac8e30101160807a22cfc985f86@34.199.143.211:60303"  
           KeyStore = DefaultKeystore
           GethData = os.path.join(Configure.myConfig.CONFIGPATH,"GethData")
           Configure.AccountPwd = None
           Configure.Account = None
  
        # BootNode
        Result = input("BootNode [" + BootNode + "]: ")
        if Result:
           BootNode = Result
  
        # GethData Dir
        Result = input("GethData Dir [" + GethData + "]: ")
        if Result:
           GethData = Result

        # Keystore Dir
        while True:
           Result = input("Keystore Dir [" + KeyStore + "]: ")
           if Result:
              KeyStore = Result
           if os.path.isdir(KeyStore) == True:
              break;
           Result = input("Directory does not exist, create it (Y)es, (N)o, (E)xit ? : [Y] ")
           if not Result:
              Result = "Y"
           if Result.upper() == "E":
              return
           if Result.upper() == "Y":
              try:
                 os.mkdir(KeyStore)
                 break
              except:
                 print("Error creating directory")
           
        # Create Account?
        Configure.getAccount(KeyStore)
 
        # 
        # We need to prep the config with the needed assets 
        #
        Configure.copyAssets()
              
        filePath = os.path.join(Configure.myConfig.CONFIGPATH,"Config.json")
        ConfigFile = open(filePath, 'w')
        ConfigFile.writelines("{")
        ConfigFile.writelines('"GETHDATA":"' + GethData.replace("\\", "\\\\") + '",\n')
        ConfigFile.writelines('"BOOTNODE":"' + BootNode + '",\n')
        ConfigFile.writelines('"KEYSTORE":"' + KeyStore.replace("\\", "\\\\") + '",\n')
        ConfigFile.writelines('"NETWORK":"' + str(chainId) + '",\n')

        # Account Name
        ConfigFile.writelines('"ACCOUNT":"' + Configure.Account + '",\n')
    
        objPswd = Configure.encode("VmxSanDiego", Configure.AccountPwd)
        ConfigFile.writelines('"ACCOUNTPWD":"' + objPswd + '"\n')

        ConfigFile.writelines("}\n")
        ConfigFile.close()

        if Configure.noSavePath:
           return
        #
        #  Create the cookie for where this config.json lives
        #  so we can run tools on the chain without being tied
        #  to starting in the directory
        #
        try:
           homePath = os.path.join(os.environ["HOME"], ".veriteem")
        except:
           return

        if os.path.isdir(homePath) == False:
           try:
              os.mkdir(homePath)
           except Exception as ex:
              print (ex)              
 

        try:
           cookieFile = open(homePath + "/config", "w")
        except:
           return

        cookieFile.writelines(Configure.myConfig.CONFIGPATH);
        cookieFile.close()

    @classmethod
    def copyAssets(self):
        configPath = Configure.myConfig.CONFIGPATH
        if not os.path.isdir(configPath):
           try:
              os.mkdir(configPath)
           except:
              errMsg = "Unable to create " + configPath 
              raise Exception(errMsg)

        fileList = ["genesis.json","Config.json"]

        for asset in fileList:
            path = os.path.join(configPath,asset)
            if path == None:
               continue
            if not os.path.isfile(path):
               path = Configure.myConfig.getFilePath(asset)
               print("Asset=" + asset)
               print("Path=" + path)
               print("ConfigPath=" + configPath)
               shutil.copy(path, configPath)

    @classmethod
    def encode(self, key, clear):
        enc = []
        for i in range(len(clear)):
            key_c = key[i % len(key)]
            enc_c = chr((ord(clear[i]) + ord(key_c)) % 256)
            enc.append(enc_c)
        return base64.urlsafe_b64encode("".join(enc).encode()).decode()
   

    @classmethod
    def getSystem(self):

        try:
            System = Configure.System
            if System == "Permissioned":
               System = "1"
            if System == "Open":
               System = "2"
            if System == "Quorum":
               System = "1"
        except:
            System = '1'

        Result = input("Select System 1=Compliance Ledger (Permissioned) 2=Compliance Ledger (Open) 3=Quorum  (" + System + ") :")
        if not Result:
           Result = System 
 
        if Result == '1':
           return 'Permissioned'
        if Result == '2':
           return 'Open'
        if Result == '3':
           return 'Quorum'
        return 'Open'

    @classmethod
    def getAccount(self, KeyStore):

        firstPass = True
        while True :
           if firstPass == False:
              Result = input("Do you want to Exit [N] : ")
              if not Result : 
                 Result = "N"
              if Result.upper() == "Y" :
                 Configure.account = None
                 return

           firstPass = False

           if Configure.Account is not None:
              print("Current Configured Account is " + Configure.Account)
              Result = input("Use (C)urrent, (N)ew, or (O)ther Account ? [C] : ")
              if not Result:
                 Result = "C" 
           else:
              Result = input("Use (N)ew, or (O)ther Account ? [N] : ")
              if not Result :
                 Result = "N"
           
           #
           # The user wants to use the account currently configured
           #
           
           if Result.upper() == "C":
              Configure.getPassword()
              if Configure.AccountPwd is not None:
                 return
              continue

           #
           # The user wants to create a new account
           #
           if Result.upper() == "N":
              Configure.createAccount(KeyStore)
              if Configure.Account is None:
                 continue
              return
                  
           #
           # The user wants to use a provided account 
           #
           if Result.upper() != "O" :
              continue

           Result = input("Enter Account Number : ")
           if not Result :
              continue

           Configure.getPassword()
           if Configure.AccountPwd is not None:
              Configure.Account = Result
              return

        
    @classmethod
    def createAccount(self, KeyStore):

        Configure.getPassword()
        if Configure.AccountPwd is None:
           print("No password entered " + Configure.AccountPwd)
           return None

        if (os.path.exists(KeyStore) == False): 
           try:
              os.mkdir(KeyStore)
           except:
              print("KeyStore directory does not exist ->" + KeyStore)
              return
   
        chainExe = Configure.myConfig.getChainExe()
        
        passFile = os.path.join(Configure.myConfig.CONFIGPATH, "Password.txt")
        logAcct  = os.path.join(Configure.myConfig.CONFIGPATH, "account.txt") 
        logErr   = os.path.join(Configure.myConfig.CONFIGPATH, "accterr.txt")

        Cmd = chainExe + ' account new --password ' + passFile +  ' --keystore ' + KeyStore + ' >' + logAcct + ' 2>' + logErr
        print (Cmd)
        try:
            os.system(Cmd)
        except:
            print ("Error launching geth account creation")
            return

        AcctFile = open(logAcct, "r")
        AccountInfo = AcctFile.read()
    
        for idx in range(0, len(AccountInfo)):
            if AccountInfo[idx] == '{':
               startIdx = idx + 1
            if AccountInfo[idx] == '}':
               endIdx = idx

        Configure.Account = "0x" + AccountInfo[startIdx:endIdx]
        print("Account = " + Configure.Account)

        os.remove(passFile)

    @classmethod
    def getPassword(self):

        print("Collecting password")
        done = False
        while done == False:
 
          AccountPwd = getpass.getpass(prompt="Enter Account Password : ")
          if not AccountPwd:
             print("Must enter password")
             Response = input("Do you want to quit? [N] ")
             if not Response :
                Response = "N"
             if Response.upper() == "Y":
                return None
             continue

          Confirm = getpass.getpass("Confirm Account Password  : ")
          if not Confirm:
             print("Must enter password")
             continue
          if AccountPwd == Confirm:
             done = True
          else:
             print("Entered passwords do not match ")

        passFile = os.path.join(Configure.myConfig.CONFIGPATH,"Password.txt")
        PasswordFile = open(passFile, "w")
        PasswordFile.writelines(AccountPwd + "\n")
        PasswordFile.close()

        Configure.AccountPwd = AccountPwd
